package ru.skipy.tests.ui.border;

import javax.swing.border.Border;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;

/**
 * FootBorder - sample border
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since Sep 21, 2010
 */
public class FootBorder implements Border {

    private static final int BORDER_THICKNESS = 19;

    private static final int GAP;

    static {
        GAP = 1;
    }

    /**
     * Foot image width
     */
    private static final int FOOT_W = 8;

    /**
     * foot image height
     */
    private static final int FOOT_H = 16;

    /**
     * Left/Right foots images for all directions
     */
    private BufferedImage leftN = new BufferedImage(FOOT_W, FOOT_H, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage rightN = new BufferedImage(FOOT_W, FOOT_H, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage leftE = new BufferedImage(FOOT_H, FOOT_W, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage rightE = new BufferedImage(FOOT_H, FOOT_W, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage leftS = new BufferedImage(FOOT_W, FOOT_H, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage rightS = new BufferedImage(FOOT_W, FOOT_H, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage leftW = new BufferedImage(FOOT_H, FOOT_W, BufferedImage.TYPE_INT_ARGB);
    private BufferedImage rightW = new BufferedImage(FOOT_H, FOOT_W, BufferedImage.TYPE_INT_ARGB);

    public FootBorder() {
        Image left = Toolkit.getDefaultToolkit().createImage(FootBorder.class.getResource("/foot_left.png"));
        Image right = Toolkit.getDefaultToolkit().createImage(FootBorder.class.getResource("/foot_right.png"));
        RenderingHints hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        AffineTransform transform = new AffineTransform(0, 1, -1, 0, 16, 0);
        AffineTransformOp operation = new AffineTransformOp(transform, hints);
        Graphics g = leftN.getGraphics();
        while (!g.drawImage(left, 0, 0, null)) {
        }
        g = rightN.getGraphics();
        while (!g.drawImage(right, 0, 0, null)) {
        }
        operation.filter(leftN, leftE);
        operation.filter(rightN, rightE);
        transform = new AffineTransform(0, 1, -1, 0, 8, 0);
        operation = new AffineTransformOp(transform, hints);
        operation.filter(leftE, leftS);
        operation.filter(rightE, rightS);
        transform = new AffineTransform(0, 1, -1, 0, 16, 0);
        operation = new AffineTransformOp(transform, hints);
        operation.filter(leftS, leftW);
        operation.filter(rightS, rightW);
    }


    @Override
    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
        Graphics2D g2d = (Graphics2D) g;
        Paint p = g2d.getPaint();

        Area border = new Area(new Rectangle(x, y, width - 1, height - 1));
        border.subtract(new Area(new Rectangle(x + BORDER_THICKNESS - 1, y + BORDER_THICKNESS - 1, width - 2 * BORDER_THICKNESS + 1, height - 2 * BORDER_THICKNESS + 1)));
        g2d.setPaint(Color.gray);
        g2d.draw(border);
        int stepsX = (width - 2 * GAP) / (FOOT_H + GAP);
        int stepsY = (height - 2 * FOOT_W - 4 * GAP) / (FOOT_H + GAP);

        double stepSizeX = (double) (width - 2 * GAP) / (double) stepsX;
        double stepSizeY = (double) (height - 2 * FOOT_W - 4 * GAP) / (double) stepsY;
        for (int i = 0; i < stepsX; i++) {
            g2d.drawImage(leftE, x + (int) (stepSizeX * i) + GAP + (int) (stepSizeX - FOOT_H) / 2, y + GAP, c);
            if (i < stepsX - 1)
                g2d.drawImage(rightE, x + 2 * GAP + (int) (stepSizeX * (i + 0.5)) + (int) (stepSizeX - FOOT_H) / 2, y + 2 * GAP + FOOT_W, c);

            g2d.drawImage(leftW, x + (int) (stepSizeX * i) + GAP + (int) (stepSizeX - FOOT_H) / 2, y + GAP + height - BORDER_THICKNESS + FOOT_W + 1, c);
            if (i < stepsX - 1)
                g2d.drawImage(rightW, x + 2 * GAP + (int) (stepSizeX * (i + 0.5)) + (int) (stepSizeX - FOOT_H) / 2, y + height - BORDER_THICKNESS + 1, c);
        }

        for (int i = 0; i < stepsY; i++) {
            g2d.drawImage(leftS, x + width - FOOT_W - 2 * GAP + 1, y + FOOT_W + 2 * GAP + (int) (stepSizeY * i) + (int) (stepSizeY - FOOT_H) / 2, c);
            if (i < stepsY - 1)
                g2d.drawImage(rightS, x + width - BORDER_THICKNESS + 1, y + FOOT_W + 3 * GAP + (int) (stepSizeY * (i + 0.5)) + (int) (stepSizeY - FOOT_H) / 2, c);
            g2d.drawImage(leftN, x + GAP, y + FOOT_W + 2 * GAP + (int) (stepSizeY * i) + (int) (stepSizeY - FOOT_H) / 2, c);
            if (i < stepsY - 1)
                g2d.drawImage(rightN, x + FOOT_W + 2 * GAP, y + FOOT_W + 3 * GAP + (int) (stepSizeY * (i + 0.5)) + (int) (stepSizeY - FOOT_H) / 2, c);
        }

        g2d.setPaint(p);
    }

    @Override
    public Insets getBorderInsets(Component c) {
        return new Insets(BORDER_THICKNESS, BORDER_THICKNESS, BORDER_THICKNESS, BORDER_THICKNESS);
    }

    @Override
    public boolean isBorderOpaque() {
        return false;
    }
}
