package ru.skipy.tests.ui.border;

import javax.swing.border.Border;
import java.awt.*;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;

/**
 * Gradient border implementation. Since colors (arbitrary) are supplied by user,
 * border is not opaque.
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 22.12.2010
 */
public class GradientBorder implements Border {

    /**
     * Border thickness
     */
    private int thickness;

    /**
     * Top-left corner color
     */
    private Color colorTL;

    /**
     * Bottom-down corner color
     */
    private Color colorBR;

    /**
     * Creates border
     *
     * @param thickness border thickness
     * @param colorTL   color of the gradient in the top-left corner
     * @param colorBR   color of the gradient in the bottom-right corner
     */
    public GradientBorder(int thickness, Color colorTL, Color colorBR) {
        this.thickness = thickness;
        this.colorTL = colorTL;
        this.colorBR = colorBR;
    }

    /**
     * Paints border
     *
     * @param c      component
     * @param g      graphic context
     * @param x      x-coordinate of top-left corner of the border
     * @param y      y-coordinate of top-left corner of the border
     * @param width  width of the enclosing rectangle of the border
     * @param height height of the enclosing rectangle of the border
     */
    @Override
    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
        if (!(g instanceof Graphics2D))
            return;
        Area border = new Area(new Rectangle2D.Float(x, y, width, height));
        border.subtract(
                new Area(new Rectangle2D.Float(x + thickness, y + thickness,
                        width - 2 * thickness, height - 2 * thickness)));
        Paint gradient = new GradientPaint(x, y, colorTL, x + width, y + height, colorBR);
        Graphics2D g2d = (Graphics2D) g;
        Paint p = g2d.getPaint();
        g2d.setPaint(gradient);
        g2d.fill(border);
        g2d.setPaint(p);
    }

    /**
     * Returns border insets for the component specified
     *
     * @param c component
     * @return border insets
     */
    @Override
    public Insets getBorderInsets(Component c) {
        return new Insets(thickness, thickness, thickness, thickness);
    }

    /**
     * Returns <code>opaque</code> property value
     *
     * @return always <code>false</code>
     */
    @Override
    public boolean isBorderOpaque() {
        return false;
    }
}
