package ru.skipy.tests.ui.delegate;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.List;

/**
 * ComponentContainer
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 20.12.2010
 */
public class ComponentContainer extends JPanel {

    /**
     * Component renderer factory instance to create renderer for the new component
     */
    private ComponentRendererFactory rendererFactory;

    /**
     * Component selection listener
     */
    private ComponentSelectionListener selectionListener;

    /**
     * Components list
     */
    private List<ComponentPanel> components;

    /**
     * Creates component container
     */
    public ComponentContainer() {
        super();
        setLayout(null);
        ComponentCreator cc = new ComponentCreator();
        addMouseListener(cc);
        addMouseMotionListener(cc);
        components = new LinkedList<ComponentPanel>();
        selectionListener = new SelectionListener();
        setBackground(Color.white);
    }

    /**
     * Sets renderer factory
     *
     * @param rendererFactory renderer factory instance
     */
    public void setRendererFactory(ComponentRendererFactory rendererFactory) {
        this.rendererFactory = rendererFactory;
    }

    /**
     * Mouse actions handler
     */
    private class ComponentCreator extends MouseAdapter {

        private ComponentPanel currentComponent = null;

        private int startX;
        private int startY;

        @Override
        public void mousePressed(MouseEvent e) {
            if (rendererFactory == null)
                return;
            currentComponent = new ComponentPanel(rendererFactory.createRenderer());
            add(currentComponent);

            startX = e.getX();
            startY = e.getY();
            fetchCurrentComponent(e);
            currentComponent.setBorder(BorderFactory.createLineBorder(Color.gray));
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            if (e.getX() == startX && e.getY() == startY) {
                remove(currentComponent);
                return;
            }
            fetchCurrentComponent(e);
            components.add(currentComponent);
            currentComponent.setBorder(null);
            currentComponent.addComponentSelectionListener(selectionListener);
            currentComponent.setSelected(true);
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            fetchCurrentComponent(e);
            repaint();
        }

        private void fetchCurrentComponent(MouseEvent e) {
            int endX = e.getX();
            int endY = e.getY();
            currentComponent.setBounds(Math.min(startX, endX),
                    Math.min(startY, endY),
                    Math.abs(startX - endX),
                    Math.abs(startY - endY));
        }

    }

    /**
     * Component selection listener implementation
     */
    private class SelectionListener implements ComponentSelectionListener {
        public void componentSelected(ComponentPanel c) {
            for (ComponentPanel panel : components) {
                if (panel != c) {
                    panel.setSelected(false);
                }
            }
            repaint();
        }
    }

}
